package com.jiuqi.util;

import java.util.Comparator;
import java.util.ArrayList;

public class RecIndex
{
    protected boolean[] descfields;
    protected int lastFound;
    protected ArrayList keys;
    protected ArrayList values;
    private Comparator keyComparator;
    
    public RecIndex() {
        this.lastFound = -1;
        this.keys = new ArrayList(128);
        this.values = new ArrayList(128);
    }
    
    public static final int lengthCompare(final Object[] keys1, final Object[] keys2, final int length) {
        int ml = keys1.length;
        if (ml > keys2.length) {
            ml = keys2.length;
        }
        if (ml > length) {
            ml = length;
        }
        for (int i = 0; i < ml; ++i) {
            final int r = compare(keys1[i], keys2[i]);
            if (r != 0) {
                return r;
            }
        }
        if (ml >= length) {
            return 0;
        }
        return keys1.length - keys2.length;
    }
    
    public static final int compare(final Object rec1, final Object rec2) {
        if (rec1 instanceof Object[] && rec2 instanceof Object[]) {
            final Object[] oa1 = (Object[])rec1;
            final Object[] oa2 = (Object[])rec2;
            int ml = oa1.length;
            if (ml > oa2.length) {
                ml = oa2.length;
            }
            for (int i = 0; i < ml; ++i) {
                final int r = compare(oa1[i], oa2[i]);
                if (r != 0) {
                    return r;
                }
            }
            return oa1.length - oa2.length;
        }
        if (rec1 == null || rec2 == null) {
            return ((rec1 != null) - (rec2 != null)) ? 1 : 0;
        }
        if (rec1.equals(rec2)) {
            return 0;
        }
        try {
            if (rec1 instanceof Comparable) {
                return ((Comparable)rec1).compareTo(rec2);
            }
            if (rec2 instanceof Comparable) {
                return ((Comparable)rec2).compareTo(rec1);
            }
            return ((boolean)rec1 - (boolean)rec2) ? 1 : 0;
        }
        catch (final ClassCastException ex) {
            return 0;
        }
    }
    
    public static final boolean matchKey(final Object pattern, final Object key) {
        if (pattern == null) {
            return true;
        }
        if (key == null) {
            return false;
        }
        if (pattern.equals(key)) {
            return true;
        }
        if (!(pattern instanceof Object[]) || !(key instanceof Object[])) {
            return false;
        }
        final int len = ((Object[])pattern).length;
        if (len != ((Object[])key).length) {
            return false;
        }
        for (int i = 0; i < len; ++i) {
            if (((Object[])pattern)[i] != null && !((Object[])pattern)[i].equals(((Object[])key)[i])) {
                return false;
            }
        }
        return true;
    }
    
    public void setDescFields(final boolean[] value) {
        this.descfields = value;
    }
    
    public void setCapacity(final int capacity) {
        this.keys.ensureCapacity(capacity);
        this.values.ensureCapacity(capacity);
    }
    
    public void setKeyComparator(final Comparator comparator) {
        this.keyComparator = comparator;
    }
    
    public final int compareKey(final Object rec1, final Object rec2) {
        if (this.keyComparator != null) {
            return this.keyComparator.compare(rec1, rec2);
        }
        if (this.descfields == null) {
            return compare(rec1, rec2);
        }
        if (rec1 instanceof Object[] && rec2 instanceof Object[]) {
            final Object[] oa1 = (Object[])rec1;
            final Object[] oa2 = (Object[])rec2;
            int ml = oa1.length;
            if (ml < oa2.length) {
                ml = oa2.length;
            }
            for (int i = 0; i < ml; ++i) {
                int ret = compare(oa1[i], oa2[i]);
                if (ret != 0) {
                    if (this.descfields[i]) {
                        ret = -ret;
                    }
                    return ret;
                }
            }
            return (oa1.length > ml) ? 1 : ((oa2.length > ml) ? -1 : 0);
        }
        int ret2 = compare(rec1, rec2);
        if (this.descfields[0]) {
            ret2 = -ret2;
        }
        return ret2;
    }
    
    public void clear() {
        this.keys.clear();
        this.values.clear();
        this.lastFound = -1;
    }
    
    public final int size() {
        return this.keys.size();
    }
    
    public final Object getKey(final int index) {
        return this.keys.get(index);
    }
    
    public final Object getValue(final int index) {
        return this.values.get(index);
    }
    
    public final void setValue(final int index, final Object value) {
        this.values.set(index, value);
    }
    
    public final int search(final Object key) {
        if (this.lastFound >= 0 && this.compareKey(key, this.keys.get(this.lastFound)) == 0) {
            return this.lastFound;
        }
        int sb = 0;
        int se = this.keys.size() - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            flag = this.compareKey(key, this.keys.get(sp));
            if (flag > 0) {
                sb = sp + 1;
            }
            else {
                if (flag >= 0) {
                    sb = sp;
                    break;
                }
                se = sp - 1;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public final int searchfirst(final Object key) {
        int sb = 0;
        int se = this.keys.size() - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            flag = this.compareKey(key, this.keys.get(sp));
            if (flag > 0) {
                sb = sp + 1;
            }
            else if (flag < 0) {
                se = sp - 1;
            }
            else {
                if (sb == (se = sp)) {
                    break;
                }
                continue;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public final int searchlast(final Object key) {
        int sb = 0;
        int se = this.keys.size() - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se + 1 >> 1;
            flag = this.compareKey(key, this.keys.get(sp));
            if (flag > 0) {
                sb = sp + 1;
            }
            else if (flag < 0) {
                se = sp - 1;
            }
            else {
                if (se == (sb = sp)) {
                    break;
                }
                continue;
            }
        }
        return (flag == 0) ? sb : (-sb - 1);
    }
    
    public final int partlySearch(final Object[] partKey, final IntValue count) {
        int sb = 0;
        int lastEqual = 0;
        int lastLarger;
        int se = lastLarger = this.keys.size() - 1;
        int flag = -1;
        while (sb <= se) {
            final int sp = sb + se >> 1;
            flag = lengthCompare(partKey, this.keys.get(sp), partKey.length);
            if (flag > 0) {
                sb = sp + 1;
            }
            else if (flag < 0) {
                se = (lastLarger = sp - 1);
            }
            else {
                lastEqual = sp;
                if (sb == (se = sp)) {
                    break;
                }
                continue;
            }
        }
        if (flag != 0) {
            count.value = 0;
            return -sb - 1;
        }
        se = lastLarger;
        lastLarger = sb;
        sb = lastEqual;
        while (sb <= se) {
            final int sp = sb + se + 1 >> 1;
            flag = lengthCompare(partKey, this.keys.get(sp), partKey.length);
            if (flag > 0) {
                sb = sp + 1;
            }
            else if (flag < 0) {
                se = sp - 1;
            }
            else {
                if (se == (sb = sp)) {
                    break;
                }
                continue;
            }
        }
        count.value = sb - lastLarger + 1;
        return lastLarger;
    }
    
    public final boolean haskey(final Object key) {
        return this.search(key) >= 0;
    }
    
    public final int indexOf(final Object key) {
        final int index = this.search(key);
        return (index < 0) ? -1 : index;
    }
    
    public final Object findkey(final Object key) {
        final int index = this.search(key);
        if (index < 0) {
            return null;
        }
        this.lastFound = index;
        return this.values.get(index);
    }
    
    public final int findfirstkey(final Object key) {
        final int index = this.searchfirst(key);
        if (index < 0) {
            return -1;
        }
        return index;
    }
    
    public final int findlastkey(final Object key) {
        final int index = this.searchlast(key);
        if (index < 0) {
            return -1;
        }
        return index;
    }
    
    public void add(final Object key) throws DuplicationException {
        this.add(key, new Integer(this.keys.size()));
    }
    
    public void add(final Object key, final Object value) throws DuplicationException {
        int index = this.search(key);
        if (index < 0) {
            index = -index - 1;
            this.keys.add(index, key);
            this.values.add(index, value);
            this.lastFound = index;
            return;
        }
        if (this.values.get(index) == value) {
            return;
        }
        throw new DuplicationException(("键值重复：" + key == null) ? null : key.toString());
    }
    
    public int addNew(final Object key) {
        return this.addNew(key, new Integer(this.keys.size()));
    }
    
    public int addNew(final Object key, final Object value) {
        int index = this.search(key);
        if (index < 0) {
            index = -index - 1;
            this.keys.add(index, key);
            this.values.add(index, value);
            this.lastFound = index;
        }
        return index;
    }
    
    public void remove(final int index) {
        if (index < this.lastFound) {
            --this.lastFound;
        }
        if (index == this.lastFound) {
            this.lastFound = -1;
        }
        this.keys.remove(index);
        this.values.remove(index);
    }
    
    public boolean remove(final Object key) {
        final int index = this.search(key);
        if (index >= 0) {
            this.keys.remove(index);
            this.values.remove(index);
            if (index < this.lastFound) {
                --this.lastFound;
            }
            if (index == this.lastFound) {
                this.lastFound = -1;
            }
        }
        return index >= 0;
    }
    
    public final int addfirst(final Object key) {
        return this.addfirst(key, new Integer(this.keys.size()));
    }
    
    public final int addfirst(final Object key, final Object value) {
        int index = this.searchfirst(key);
        if (index < 0) {
            index = -index - 1;
        }
        this.keys.add(index, key);
        this.values.add(index, value);
        return this.lastFound = index;
    }
    
    public final boolean removefirst(final Object key) {
        final int index = this.searchfirst(key);
        if (index >= 0) {
            this.keys.remove(index);
            this.values.remove(index);
            if (index < this.lastFound) {
                --this.lastFound;
            }
            if (index == this.lastFound) {
                this.lastFound = -1;
            }
        }
        return index >= 0;
    }
    
    public final int addlast(final Object key) {
        return this.addlast(key, new Integer(this.keys.size()));
    }
    
    public final int addlast(final Object key, final Object value) {
        int index = this.searchlast(key);
        if (index >= 0) {
            ++index;
        }
        else {
            index = -index - 1;
        }
        this.keys.add(index, key);
        this.values.add(index, value);
        return this.lastFound = index;
    }
    
    public final boolean removelast(final Object key) {
        final int index = this.searchlast(key);
        if (index >= 0) {
            this.keys.remove(index);
            this.values.remove(index);
            if (index < this.lastFound) {
                --this.lastFound;
            }
            if (index == this.lastFound) {
                this.lastFound = -1;
            }
        }
        return index >= 0;
    }
    
    public final int indexOfValue(final Object value) {
        return this.values.indexOf(value);
    }
}
